/**************************************************************************//**
 * @file     main.c
 * @brief    Main program body
 * @version  V1.1.0
 * @date     14-Feb-2019
 *
 * @note
 * Copyright (C) 2019 Spintrol Limited Corporation. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING 
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE 
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE 
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/

#include "spd1188.h"
#include <stdio.h>


uint8_t                     u8PREDRIID                                    = 0;                 /* PRE-DRIVER mode ID */
uint8_t                     u8PREDRIDATA;                                                       /* Data read from PRE-DRIVER */
ErrorStatus                 eErrorState;
uint32_t                    u32SSPBaundrate                               = 1000000;           /* The baundrate of SSP communicating with PRE-DRIVER mode */
uint32_t                    u32Timeout                                    = 500;               /* The number of times try to get PRE-DRIVER mode ID */
uint8_t                     Count;


 /*************************************************************************************************************************
 *
 * @brief      This case show how to set the BUCK3 FREQ as a fixed value(BUCK3 is in PFM mode bu default).
 *
 *                                                                 VDDG(12V by default,100~200mA)
 *                                                                 / \
 *                                                                  |       SPD1188  Chip
 *                                        --------------------------|----------------------------
 *                                       |    High Voltage Part     |                            |
 *                                       |    -----------------     |                            |
 *                                       |   |      -------    |    |                            |
 *                                VBAT---|---|---->|       |---|----                             |        SW3
 *                                       |   |     | BUCKG | | |                                 |        ---
 *                                       |   |      -------  | |                                 |-------| X |
 *                                       |   |          -----  |            Low voltage part     |        ---
 *                                       |   |         |       |          --------------------   |
 *                                       |   |        \ /      |         |                    |  |
 *                                       |   |      -------    |   1.2V  |                    |  |
 *                                       |   |     |       |---|-------->|         MCU        |  |
 *                                       |   |     | BUCK3 |---|----     |                    |  |
 *                                       |   |     |       |   |    |    |                    |  |
 *                                       |   |      -------    |    |    |                    |  |
 *                                       |    -----------------     |     --------------------   |
 *                                        --------------------------|----------------------------
 *                                                                  |
 *                                                                 \ /
 *                                                                 VDDC(3.3V,500mA)
 *
 *             If the circuit is open, you can measure the BUCKG waveform with osciloscope connected to SW3, there give a
 *             diagram of the BUCK3 waveform. you can use the formula below to check the the BUCK3 FREQ setting is working
 *             or not.(If the waveform is too messy, you can force BUCK3 into PWM mode, but PFM is recommanded.)
 *
 *                              |<-Ton->|
 *             -----             -------     ---                -------     ---
 *              / \             |       |   |   |   -   -      |       |   |   |   -   -
 *               |   ---   -----        |   |   |  | | | ------        |   |   |  | | | -----BUCK3 FREQ amplitude is 0~VDDG
 *               |   / \                |   |   |  | | |               |   |   |  | | |
 *           VDDG|    |                 |   |   |  | | |               |   |   |  | | |
 *               |    |3.3              |   |   |  |  -                |   |   |  |  -
 *               |    |                 |   |    --                    |   |    --
 *              \ /  \ /                |   |                          |   |
 *             ----- ---                 ---                            ---
 *
 *             Formula for Ton :
 *                              Ton = T * (3.3 / VDDG)      ('T' is the corresponding period of the BUCK3 FREQ)
 *             Key point :
 *                        There are three steps for engineering tunning in the following demo code, these code is just for
 *                        test silicon, we will delete these code in real production.
 *             Key point:
 *                 PRE-DRIVER init include two step
 *                     <1> Init the communication circuits(SSP circyits) with MCU, we call this step 'Init PRE-DRIVER module'.
 *                     <2> Init PRE-DRIVER socket itself, we call this step 'Power up PRE-DRIVER mode'.
 *
 *************************************************************************************************************************/


int main(void)
{
  FLASH_WALLOW();
  FLASH_SetTiming(200000000);
  /* Disable flash write access after flash operation had done */
  FLASH_WDIS();

  CLOCK_InitWithRCO(CLOCK_HCLK_200MHZ);

  Delay_Init();

  /*
   * Init the UART
   *
   * 1.Set the GPIO34/35 as UART FUNC
   *
   * 2.Enable the UART CLK
   *
   * 3.Set the rest para
   */
  GPIO_SetPinChannel(GPIO_34,GPIO34_UART_TXD);
  GPIO_SetPinChannel(GPIO_35,GPIO35_UART_RXD);
  CLOCK_EnableModule(UART_MODULE);
  UART_Init(UART,38400);

  /* Init PRE-DRIVER module(this step means 'Init PRE-DRIVER module') and get the ID */
  eErrorState = HV_Init(u32SSPBaundrate, &u8PREDRIID, u32Timeout);
  if(eErrorState == ERROR)
  {
    printf("Init PRE-DRIVER mode FAIL\n");
    return 0;
  }
  else
  {
    printf("Init PRE-DRIVER mode SUCCESS[ID:%d]\n",u8PREDRIID);
  }

  /* Read the value BUCK3CTL had haven */
  eErrorState = HV_ReadReg(HV_REG_BUCK3CTL, &u8PREDRIDATA);
  if(eErrorState == ERROR)
  {
    printf("Read BUCK3CTL FAIL\n");
    return 0;
  }

  /* 
   * Fixed BUCK3 FREQ as 400KHz.
   * Before you set the corresponding bits to a new value, you need to set these bits to zero, because there may be some bits is not
   * zero, and if you do and operation with these value, you may get a unanticipated value.
   */
  eErrorState = HV_WriteReg(HV_REG_BUCK3CTL, (u8PREDRIDATA & (~(0x3UL << BUCK3CTL_ALL_FREQSEL_Pos))) | BUCK3CTL_ALL_FREQSEL_400_kHz);
  if(eErrorState == ERROR)
  {
    printf("Set BUCK3 FREQ FAIL\n");
    return ERROR;
  }

  /* PDRVCTL register can only be writable when CTLKEY=0x99 */
  eErrorState = HV_WriteReg(HV_REG_CTLKEY, CTLKEY_ALL_KEY_BUCKEN);
  if(eErrorState == ERROR)
  {
    printf("Write CTLKEY register FAIL\n");
    return 0;
  }

  /* Power up PRE-DRIVER mode(this step means 'Power up PRE-DRIVER mode') */
  eErrorState = HV_WriteReg(HV_REG_PDRVCTL, PDRVCTL_ALL_EN_ENABLE);
  if(eErrorState == ERROR)
  {
    printf("Power up PRE-DRIVER FAIL\n");
    return 0;
  }

  /* Delay 100us to wait for the VDDG LDO ready */
  Delay_Us(100);
  
  /* Wait for PRE-DRIVER mode become enable */
  while(((u8PREDRIDATA & (0x1 << 0)) == 0) && (Count < 10)){
    eErrorState = HV_ReadReg(HV_REG_GLBSTS0, &u8PREDRIDATA);
    if(eErrorState == ERROR)
    {
      printf("PRE-DRIVER enable FAIL[%02x]\n",u8PREDRIDATA);
      return 0;
    }
    if(Count > 10)
    {
      printf("PRE-DRIVER enable timeout\n");
      return 0;
    }
    Count ++;
  }
  
  /* additional delay after VDDG ready, for final power settling */
  Delay_Us(100);

  while(1)
  {

  }
}
