/******************************************************************************
 * @file     main.c
 * @brief    Main program body
 * @version  V1.8.0
 * @date     31-Dec-2021
 *
 * @note
 * Copyright (C) 2021 Spintrol Limited Corporation. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/

#include"spc1168.h"
#include<stdio.h>

/* This macro is used to get the PWM period with a specified frequency */
#define                PWMPeriod(u32PWMFreqHz)             ((CLOCK_GetModuleClock(PWM_MODULE))/u32PWMFreqHz)/2;

#define                PWM_FREQ                            20000                  /* 20kHz PWM */
#define                PWM_DB_NS                           1000                   /* 1ns */
#define                PWM_SYNC_RELOADVAL                  0x0                    /* the value load to TBCNT after sync signal */

uint32_t u32PWMPeriod;
uint32_t u32RELoadValA;


 /*************************************************************************************************************************
 *
 * @brief      In this case, PWM0 will output SYNC signal triggered by software force, so there is nothing to do with 
 *             time-based counter, in other words, there is nothing to do with counter CLK any more, so there is no need to
 *             care about the counter CLK of these PWMs. 
 * 
 *             The key point of this demo is keeping the same counting mode and SYNC re-load value in these PWMs .
 *
 *************************************************************************************************************************/


int main()
{ 
  FLASH_WALLOW();
  FLASH_SetTiming(200000000);
  /* Disable flash write access after flash operation had done */
  FLASH_WDIS();

  CLOCK_InitWithRCO(CLOCK_HCLK_200MHZ);

  Delay_Init();

  /*
   * Init the UART
   *
   * 1.Set the GPIO34/35 as UART FUNC
   *
   * 2.Enable the UART CLK
   *
   * 3.Set the rest para
  */
  GPIO_SetPinChannel(GPIO_34,GPIO34_UART_TXD);
  GPIO_SetPinChannel(GPIO_35,GPIO35_UART_RXD);
  CLOCK_EnableModule(UART_MODULE);
  UART_Init(UART,38400);

  /* Select GPIO30/31 as the channel A/B output of PWM0 respectively */
  GPIO_SetPinChannel(GPIO_30,GPIO30_PWM0A);
  GPIO_SetPinChannel(GPIO_31,GPIO31_PWM0B);

  /* Select GPIO36/37 as the channel A/B output of PWM1 respectively */
  GPIO_SetPinChannel(GPIO_36,GPIO36_PWM1A);
  GPIO_SetPinChannel(GPIO_37,GPIO37_PWM1B);

  /* Select GPIO38/39 as the channel A/B output of PWM2 respectively */
  GPIO_SetPinChannel(GPIO_38,GPIO38_PWM2A);
  GPIO_SetPinChannel(GPIO_39,GPIO39_PWM2B);

  /* Init PWM0/1/2 and output 20kHz waveform on both channel A and channel B */
  PWM_ComplementaryPairChannelInit(PWM0, PWM_FREQ, PWM_DB_NS);
  PWM_ComplementaryPairChannelInit(PWM1, PWM_FREQ, PWM_DB_NS);
  PWM_ComplementaryPairChannelInit(PWM2, PWM_FREQ, PWM_DB_NS);

  /* Set PWM0A output 25% duty waveform */
  u32PWMPeriod = PWMPeriod(PWM_FREQ);
  PWM_SetCMPA(PWM0, (u32PWMPeriod * 3)/ 4);
  PWM_SetCMPA(PWM1, (u32PWMPeriod * 3)/ 4);
  PWM_SetCMPA(PWM2, (u32PWMPeriod * 3)/ 4);

  /*Start counting*/
  PWM_RunCounter(PWM0);
  PWM_RunCounter(PWM1);
  PWM_RunCounter(PWM2);

  /*
   * In order to generate a sotfware force SYNC signal, we need to set 'SYNCOSEL' of PWM0 to 
   * 'TBCTL_BIT_SYNCOSEL_SYNCI_AND_FRCSYNC'. In order to pass through the same SYNC signal of
   * PWM0 to the PWM2, we also need to set PWM1 as 'TBCTL_BIT_SYNCOSEL_SYNCI_AND_FRCSYNC'.
   *
   *
   * Key point : the SYNC signal can only pass through two group(from 0 to 2 and 3 to 5, 2 to
   * 3 is not permitted).
   */
  PWM_SetSyncOutEvent(PWM0,TBCTL_BIT_SYNCOSEL_SYNCI_AND_FRCSYNC);
  PWM_SetSyncOutEvent(PWM1,TBCTL_BIT_SYNCOSEL_SYNCI_AND_FRCSYNC);

  /* Enable load SYNC value from SYNC signal is happeded*/
  PWM_EnableSync(PWM0);
  PWM_EnableSync(PWM1);
  PWM_EnableSync(PWM2);

  /* Set PWMx counting down after SYNC */
  PWM_SetCounterDirAfterSync(PWM0, COUNT_DOWN);
  PWM_SetCounterDirAfterSync(PWM1, COUNT_DOWN);
  PWM_SetCounterDirAfterSync(PWM2, COUNT_DOWN);

  /*
   * Set the value will be loaded to TBCNT after SYNC signal.
   *
   * All the PWMs must be the same SYNC reload value to get the synchronized
   * waveform.
   */
  PWM_SetSyncReloadValue(PWM0, PWM_SYNC_RELOADVAL);
  PWM_SetSyncReloadValue(PWM1, PWM_SYNC_RELOADVAL);
  PWM_SetSyncReloadValue(PWM2, PWM_SYNC_RELOADVAL);

  /* Force a sofrware SYNC signal of PWM0 */
  PWM_ForceSync(INC_PWM0);

  while(1)
  {

  }
}
