/******************************************************************************
 * @file     main.c
 * @brief    Main program body
 * @version  V1.8.0
 * @date     31-Dec-2021
 *
 * @note
 * Copyright (C) 2021 Spintrol Limited Corporation. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/

#include"spc1168.h"
#include<stdio.h>

/* This macro is used to get the PWM period with a specified frequency */
#define                PWMPeriod(u32PWMFreqHz)          ((CLOCK_GetModuleClock(PWM_MODULE))/u32PWMFreqHz)/2;

#define                PWM_FREQ                          20000        /* 20kHz PWM */
#define                PWM_DB_US                         1000         /* 1us */
#define                PWM_SYNC_RELOADVAL                100          /* the value load to TBCNT after sync signal */

uint32_t               u32PWMPeriod;


 /*************************************************************************************************************************
 *
 * @brief      In this case, GPIO asserted SYNC signal will be asserted on all PWMs(PWM0/1/2) synchronously, so there is
 *             nothing to do with time-based counter, in other words, there is nothing to do with counter CLK any more, 
 *             there is no need to care about the counter CLK of these PWMs, the key point of this demo is keeping the same
 *             counting mode and SYNC re-load value in these PWMs(PWM0/1/2).
 *
 *             Key point : the SYNC GPIO(GPIO32) must be keep low level before test begin, and switch to high level when you
 *                         want to trigger the sync signal.
 *
 *************************************************************************************************************************/


int main()
{ 
  FLASH_WALLOW();
  FLASH_SetTiming(200000000);
  /* Disable flash write access after flash operation had done */
  FLASH_WDIS();

  CLOCK_InitWithRCO(CLOCK_HCLK_200MHZ);

  Delay_Init();

  /*
   * Init the UART
   *
   * 1.Set the GPIO34/35 as UART FUNC
   *
   * 2.Enable the UART CLK
   *
   * 3.Set the rest para
   */
  GPIO_SetPinChannel(GPIO_34,GPIO34_UART_TXD);
  GPIO_SetPinChannel(GPIO_35,GPIO35_UART_RXD);
  CLOCK_EnableModule(UART_MODULE);
  UART_Init(UART,38400);

  /* Select GPIO30/31 as the channel A/B output of PWM0 respectively */
  GPIO_SetPinChannel(GPIO_30,GPIO30_PWM0A);
  GPIO_SetPinChannel(GPIO_31,GPIO31_PWM0B);

  /* Select GPIO36/37 as the channel A/B output of PWM1 respectively */
  GPIO_SetPinChannel(GPIO_36,GPIO36_PWM1A);
  GPIO_SetPinChannel(GPIO_37,GPIO37_PWM1B);

  /* Select GPIO38/39 as the channel A/B output of PWM2 respectively */
  GPIO_SetPinChannel(GPIO_38,GPIO38_PWM2A);
  GPIO_SetPinChannel(GPIO_39,GPIO39_PWM2B);

  /*
   * Init PWM0/1/2 and output 20kHz waveform on both channel A and channel B.
   *
   * Key point : in this function, we init the TBCNT SYNC out signal when TBCNT=0(this
   * SYNC out signal will sent to the next PWM SINCin)
   */
  PWM_ComplementaryPairChannelInit(PWM0, PWM_FREQ, PWM_DB_US);
  PWM_ComplementaryPairChannelInit(PWM1, PWM_FREQ, PWM_DB_US);
  PWM_ComplementaryPairChannelInit(PWM2, PWM_FREQ, PWM_DB_US);

  /* If we want to use GPIO as the SYNC signal, TBCNT SYNC out signal when TBCNT=0 must be close */
  PWM_SetSyncOutEvent(PWM0, TBCTL_BIT_SYNCOSEL_DISABLE);
  PWM_SetSyncOutEvent(PWM1, TBCTL_BIT_SYNCOSEL_DISABLE);

  /* Set PWMxA output 75% duty waveform */
  u32PWMPeriod = PWMPeriod(PWM_FREQ);
  PWM_SetCMPA(PWM0, u32PWMPeriod / 4);
  PWM_SetCMPA(PWM1, u32PWMPeriod / 4);
  PWM_SetCMPA(PWM2, u32PWMPeriod / 4);

  /* Start counting */
  PWM_RunCounter(PWM0);
  /* Generate a delay between these waveform deliberately */
  Delay_Us(7);
  PWM_RunCounter(PWM1);
  Delay_Us(7);
  PWM_RunCounter(PWM2);

  /* Enbale GPIO SYNC on PWM */
  PWM_EnableSyncFromGPIO(INC_PWM0|INC_PWM1|INC_PWM2);

  /* Set the PWMs work at counting up mode after SYNC */
  PWM_SetCounterDirAfterSync(PWM0, COUNT_UP);
  PWM_SetCounterDirAfterSync(PWM1, COUNT_UP);
  PWM_SetCounterDirAfterSync(PWM2, COUNT_UP);

  /*
   * Set the value will be loaded to TBCNT after SYNC signal.
   *
   * All the PWMs must be the same SYNC reload value to get the synchronized
   * waveform.
   */
  PWM_SetSyncReloadValue(PWM0, PWM_SYNC_RELOADVAL);
  PWM_SetSyncReloadValue(PWM1, PWM_SYNC_RELOADVAL);
  PWM_SetSyncReloadValue(PWM2, PWM_SYNC_RELOADVAL);

  /* Set the SYNC GPIO to high level first */
  GPIO_WritePin(GPIO_32,GPIO_LEVEL_HIGH);

  /*
   * Synchronization event happen when GPIO32 is low, This interface will also set GPIO as input.
   *
   * Key point : Beacuse of the GPIO is pulled up internal by MCU by default. so, Setting GPIO SYNC
   *             signal to low level is recommended.
   */
  PWM_SetSyncFromGPIO(GPIO_32, GPIO_LEVEL_LOW);

  while(1)
  {
    
  }
}
